
/*
 * Startup Code for ARM
 * This file is from the startup code version for AT91SAM7Sxx.
 *
 * Assembler Startup File.
 * Here is the very beginning of execution. This file contains
 * the ordered execution of the Initialization functions and
 * of the main function.
 *
 * by: Clemens Helfmeier
 * ClemensHelfmeier {at] gmx `dot" de
 * (replace the corresponding at and dot in the email, please).
 */


#include "AT91SAM7S64.inc"

/* Size for Stacks in Bytes */
#define STACK_IM_size	(0x10*8)
	/* size of stack for Interrupt Mode (r0, r1, lr, spsr for every nested interrupt)
	 * the maximum of 8 priorities gives 8 nested interrupt level
	 */
#define STACK_FM_size	0x00		/* Fast interrupt Mode */
#define STACK_SM_size	0x00
	/* Supervisor Mode, also used by SWI; SWI is not nested (does not call itself) */
#define STACK_AM_size	0x00		/* Abort Mode */
#define STACK_UM_size	0x00		/* Undefined Mode */

/* Interrup vectors.
 * This must be placed at the very beginning of the FLASH area
 * If the interrupts should be usable after the REMAP-Command, too
 * it should also be present in the SRAM.
 */
.section	.rvectors
.org 0x0000
	B	InitReset		/* 0x00 Reset handler */
undefvec:
	B       undefvec                /* 0x04 Undefined Instruction */

	B       SWI_Handler             /* 0x08 Software Interrupt */
pabtvec:
	B       pabtvec                 /* 0x0C Prefetch Abort */
dabtvec:
	B       dabtvec                 /* 0x10 Data Abort */
rsvdvec:
	B       rsvdvec                 /* 0x14 reserved  */
	.extern IRQ_Handler
	B	IRQ_Handler
	.extern FIQ_Handler
	B	FIQ_Handler


/* The rest of this file handles the very low level part of Initialization and
 * Interrupt handling. It can be placed anywhere -> .text
 */
.section	.text


/* InitReset
 * This is the start up routine for the processor
 * It initializes the Stack and the registers
 */
	.func InitReset
InitReset:
/* S T A C K
 * the stack for this code look like this:
 * top (high addr)	END			FIQ Stack
 *			END-FIQ_size		IRQ Stack
 *			END-FIQ-IRQ		SuperVisor Mode Stack
 *			END-FIQ-IRQ-SM		Abort Mode Stack
 *			END-FIQ-IRQ-SM-AM	Undefined Mode Stack
 * bottom (low addr)	END-FIQ-IRQ-SM-AM-UM	User Mode Stack
 * The biggest stack will always be the User Mode Stack since most program code
 * is executed within that scope. The User Mode Stack is also used by the System Mode.
 *
 */
	/* initialize stack for SuperVisorMode */
	msr	CPSR_c, #(ARM_MODE_Supervisor | ARM_I_BIT | ARM_F_BIT)
	ldr	r13, =SRAM_END - STACK_IM_size - STACK_FM_size
	/* initialize stack for AbortMode */
	msr	CPSR_c, #(ARM_MODE_Abort | ARM_I_BIT | ARM_F_BIT)
	ldr	r13, =SRAM_END - STACK_IM_size - STACK_FM_size - STACK_SM_size
	/* initialize stack for UndefinedMode */
	msr	CPSR_c, #(ARM_MODE_Undefined | ARM_I_BIT | ARM_F_BIT)
	ldr	r13, =SRAM_END - STACK_IM_size - STACK_FM_size - STACK_SM_size - STACK_AM_size
	/* initialize STACK for FIQ Mode */
	msr     CPSR_c, #(ARM_MODE_FIQ | ARM_I_BIT | ARM_F_BIT)
	ldr	r13, =SRAM_END
	/* initialize STACK for IRQ Mode */
	msr     CPSR_c, #(ARM_MODE_IRQ | ARM_I_BIT | ARM_F_BIT)
	ldr	r13, =SRAM_END - STACK_FM_size
	/* initialize STACK for User Mode and turn on interrupts here, it is not possible to do that
	 * after entering user mode. you can't even change to another mode :( */
	msr     CPSR_c, #(ARM_MODE_User)
	ldr	r13, =SRAM_END - STACK_IM_size - STACK_FM_size - STACK_SM_size - STACK_AM_size - STACK_UM_size

/* R E G I S T E R
 * there are no special registers (0-register, etc) for ARM,
 * so nothing needs to be done
 */

/* I N I T I A L   D A T A
 * the initialization of SRAM data will be made in a C-Routine
 */
	.extern AT91F_InitBSS
	bl	AT91F_InitBSS

	.extern AT91F_InitLowLevel
	bl	AT91F_InitLowLevel

/* M A I N
 * call the Main routine, jump through to exit in case of return
 */
	.extern main
	bl	main
	b exit

.size   Initreset, . - InitReset
.endfunc


/* E X I T
 * this loops forever
 * could be optimized for low-power, but currently uninteresting
 */
.func exit
exit:
	b exit
.size	exit, . - exit
.endfunc

